/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client.alerters;

//---IMPORTS---
import uk.org.iscream.cms.server.client.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;
import java.io.*;
import java.util.StringTokenizer;


/**
 * This alerter delivers alerts using e-mail.
 *
 * @author  $Author: tdb $
 * @version $Id: EMail__Alerter.java,v 1.28 2004/08/01 10:40:43 tdb Exp $
 */
public class EMail__Alerter extends AlerterSkeleton {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.28 $";
    
    /**
     * A description of this alerter
     */
    public final String DESC = "Sends alerts over e-mail.";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

//---PUBLIC METHODS---

    /**
     * Implements the abstract method from the skeleton class.
     * This method will attempt to send an alert
     * message using the configured email configuration.
     *
     * @param alert the alert to send
     */
    public void sendAlert(Alert alert) {
        // get the subject and replace fields
        String subject;
        try {
            subject = _cp.getProperty(_name, "Alerter.EMail.subject");
        } catch (PropertyNotFoundException e) {
            subject = NOT_CONFIGURED;
            _logger.write(toString(), Logger.WARNING, "Alerter.EMail.subject value unavailable using default of " + subject);
        }
        subject = processAlertMessage(subject, alert);
                    
        // get the message body and replace fields
        String message;
        try {
            message = _cp.getProperty(_name, "Alerter.EMail.message");
        } catch (PropertyNotFoundException e) {
            message = NOT_CONFIGURED;
            _logger.write(toString(), Logger.WARNING, "Alerter.EMail.message value unavailable using default of " + message);
        }
        message = processAlertMessage(message, alert);
        
        // attempt to send the actual message
        try {
            // create SMTP message
            Smtp smtp = new Smtp(_cp.getProperty(_name, "Alerter.EMail.smtpServer"));
            // set our sender
            smtp.setSender(_cp.getProperty(_name, "Alerter.EMail.sender"));
            
            // list of destination addresses
            String destList = _cp.getProperty("Host."+alert.getSource(), "Alerter.EMail.destList");
            
            // set the to: list
            StringTokenizer st = new StringTokenizer(destList, ";");
            while (st.hasMoreTokens()) {
                smtp.setTo(st.nextToken());
            }
 
            // prepare to print the message            
            PrintWriter out = smtp.getOutputStream();
            out.println("Subject: "+subject+"\n");
            
            // send the message
            out.println(message);
            smtp.sendMessage();
            smtp.close();
            _logger.write(toString(), Logger.DEBUG, "Sending " + _name + " at "+ Alert.alertLevels[alert.getLevel()] + " level");
        }
        catch(IOException e) {
            _logger.write(toString(), Logger.ERROR, "Error whilst sending message: "+e);
        }
        catch(PropertyNotFoundException e) {
            _logger.write(toString(), Logger.ERROR, "Error obtaining essential configuration: "+e);
        }
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

    /**
     * Return the String representation of what the alerter does
     *
     * @return the description
     */
    public String getDescription(){
        return DESC;
    }

//---PRIVATE METHODS---
    
//---ACCESSOR/MUTATOR METHODS---
    
    /**
     * Returns the "friendly" name of this class. This
     * is simply an accessor for _name, required due to
     * inheritance issues with extending AlerterSkeleton.
     *
     * @return the friendly name
     */
    protected String getFName() {
        return _name;
    }

//---ATTRIBUTES---
    
    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    protected String _name = "EMail";
    
//---STATIC ATTRIBUTES---

}
